# -*- coding: utf-8 -*-
"""2d mimetic polynomials in the reference domain [-1,1]^2.

© mathischeap.com
"""
import numpy as np
from scipy.sparse import csr_matrix

from mp_1d_rd import node_polynomials as np1
from mp_1d_rd import edge_polynomials as ep1
from quadrature import lobatto


def node_polynomials(nodes_xi, nodes_eta, xi, eta):
    """Compute the node polynomials built at ``nodes_xi``, ``nodes_eta``
    and evaluated at grid (``xi``, ``eta``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param xi: Grid (``xi``, ``eta``) is the grid to evaluate the
        polynomials.
    :param eta: Grid (``xi``, ``eta``) is the grid to evaluate the
        polynomials.
    :return: A 2d numpy.array whose 0-dimension refers to the number of
        node polynomials and 1-dimension refers to the values
        evaluated at grid (``xi``, ``eta``).
    :example:

    >>> nodes_xi, _ = lobatto(4)
    >>> xi = np.linspace(-1, 1, 20)
    >>> nodes_eta, _ = lobatto(4)
    >>> eta = np.linspace(-1, 1, 30)
    >>> nps = node_polynomials(nodes_xi, nodes_eta, xi, eta)
    >>> print(nps.shape)
    (25, 600)
    >>> nps[0,:] # doctest: +ELLIPSIS
    array([ 1.00000000e+00,  5.51791346e-01,  2.40674949e-01,...
    """
    np_xi = np1(nodes_xi, xi)
    np_eta = np1(nodes_eta, eta)
    return np.kron(np_eta, np_xi)


def edge_polynomials(nodes_xi, nodes_eta, xi, eta):
    """Compute the edge polynomials built at ``nodes_xi``, ``nodes_eta``
    and evaluated at grid (``xi``, ``eta``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param xi: Grid (``xi``, ``eta``) is the grid to evaluate the
        polynomials.
    :param eta: Grid (``xi``, ``eta``) is the grid to evaluate the
        polynomials.
    :return: A tuple of two outputs:

        #. (numpy.array) The edge polynomials `e(xi)*l(eta)` evaluated
            at grid (``xi``, ``eta``).
        #. (numpy.array) The edge polynomials `l(xi)*e(eta)` evaluated
            at grid (``xi``, ``eta``).
    :example:

    >>> nodes_xi, _ = lobatto(4)
    >>> xi = np.linspace(-1, 1, 20)
    >>> nodes_eta, _ = lobatto(4)
    >>> eta = np.linspace(-1, 1, 30)
    >>> eps = edge_polynomials(nodes_xi, nodes_eta, xi, eta)
    >>> print(eps[0].shape)
    (20, 600)
    >>> print(eps[1].shape)
    (20, 600)
    >>> eps[0][0,:] # doctest: +ELLIPSIS
    array([ 5.00000000e+00,  3.56239977e+00,  2.39116489e+00, ...
    >>> eps[1][0,:] # doctest: +ELLIPSIS
    array([ 5.00000000e+00,  2.75895673e+00,  1.20337474e+00, ...
    """
    np_xi = np1(nodes_xi, xi)
    np_eta = np1(nodes_eta, eta)
    ep_xi = ep1(nodes_xi, xi)
    ep_eta = ep1(nodes_eta, eta)
    return np.kron(np_eta, ep_xi), np.kron(ep_eta, np_xi)


def face_polynomials(nodes_xi, nodes_eta, xi, eta):
    """Compute the face polynomials built at ``nodes_xi``, ``nodes_eta``
    and evaluated at grid (``xi``, ``eta``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param xi: Grid (``xi``, ``eta``) is the grid to evaluate the
        polynomials.
    :param eta: Grid (``xi``, ``eta``) is the grid to evaluate the
        polynomials.
    :return: A 2d numpy.array whose 0-dimension refers to the number of
        face polynomials and 1-dimension refers to the values
        evaluated at grid (``xi``, ``eta``).
    :example:

    >>> nodes_xi, _ = lobatto(4)
    >>> xi = np.linspace(-1, 1, 20)
    >>> nodes_eta, _ = lobatto(4)
    >>> eta = np.linspace(-1, 1, 30)
    >>> fps = face_polynomials(nodes_xi, nodes_eta, xi, eta)
    >>> print(fps.shape)
    (16, 600)
    >>> fps[0,:] # doctest: +ELLIPSIS
    array([ 2.50000000e+01,  1.78119988e+01,  1.19558245e+01,...
    """
    ep_xi = ep1(nodes_xi, xi)
    ep_eta = ep1(nodes_eta, eta)
    return np.kron(ep_eta, ep_xi)


if __name__ == '__main__':
    import doctest
    doctest.testmod()

