# -*- coding: utf-8 -*-
"""3d mimetic polynomials in the reference domain [-1,1]^3.

© mathischeap.com
"""
import numpy as np

from mp_1d_rd import node_polynomials as np1
from mp_1d_rd import edge_polynomials as ep1
from quadrature import lobatto


def node_polynomials(nodes_xi, nodes_eta, nodes_sigma,
                     xi, eta, sigma):
    """Compute the node polynomials built at ``nodes_xi``, ``nodes_eta``
    and evaluated at grid (``xi``, ``eta``, ``sigma``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param nodes_sigma: The nodes on which the 1d mimetic polynomials
        are built along the third axis.
    :param xi: Grid (``xi``, ``eta``, ``sigma``) is the grid to evaluate
        the polynomials.
    :param eta: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :param sigma: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :return: A 2d numpy.array whose 0-dimension refers to the number of
        node polynomials and 1-dimension refers to the values
        evaluated at grid (``xi``, ``eta``, ``sigma``).
    :example:

    >>> nodes_xi, _ = lobatto(3)
    >>> nodes_eta, _ = lobatto(3)
    >>> nodes_sigma, _ = lobatto(3)
    >>> xi = np.linspace(-1, 1, 5)
    >>> eta = np.linspace(-1, 1, 6)
    >>> sigma = np.linspace(-1, 1, 7)
    >>> nps = node_polynomials(nodes_xi, nodes_eta, nodes_sigma, xi, eta, sigma)
    >>> print(nps.shape)
    (64, 210)
    >>> nps[0,:] # doctest: +ELLIPSIS
    array([ 1.00000000e+00,  4.68750000e-02, -1.25000000e-01,...
    """
    np_xi = np1(nodes_xi, xi)
    np_eta = np1(nodes_eta, eta)
    np_sigma = np1(nodes_sigma, sigma)
    return np.kron(np_sigma, np.kron(np_eta, np_xi))


def edge_polynomials(nodes_xi, nodes_eta, nodes_sigma,
                     xi, eta, sigma):
    """Compute the edge polynomials built at ``nodes_xi``, ``nodes_eta``
    and evaluated at grid (``xi``, ``eta``, ``sigma``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param nodes_sigma: The nodes on which the 1d mimetic polynomials
        are built along the third axis.
    :param xi: Grid (``xi``, ``eta``, ``sigma``) is the grid to evaluate
        the polynomials.
    :param eta: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :param sigma: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :return: A tuple of three outputs:

        #. (numpy.array) The edge polynomials `ell` evaluated at grid
            (``xi``, ``eta``, ``sigma``).
        #. (numpy.array) The edge polynomials `lel` evaluated at grid
            (``xi``, ``eta``, ``sigma``).
        #. (numpy.array) The edge polynomials `lle` evaluated at grid
            (``xi``, ``eta``, ``sigma``).

    :example:

    >>> nodes_xi, _ = lobatto(3)
    >>> nodes_eta, _ = lobatto(3)
    >>> nodes_sigma, _ = lobatto(2)
    >>> xi = np.linspace(-1, 1, 5)
    >>> eta = np.linspace(-1, 1, 6)
    >>> sigma = np.linspace(-1, 1, 7)
    >>> eps = edge_polynomials(nodes_xi, nodes_eta, nodes_sigma, xi, eta, sigma)
    >>> print(eps[0].shape)
    (36, 210)
    >>> print(eps[1].shape)
    (36, 210)
    >>> print(eps[2].shape)
    (32, 210)
    >>> eps[0][0,:] # doctest: +ELLIPSIS
    array([ 3.00000000e+00,  9.68750000e-01, -1.25000000e-01, ...
    >>> eps[1][0,:] # doctest: +ELLIPSIS
    array([ 3.00000000e+00,  1.40625000e-01, -3.75000000e-01, ...
    >>> eps[2][0,:] # doctest: +ELLIPSIS
    array([ 1.50000000e+00,  7.03125000e-02, -1.87500000e-01, ...
    """
    np_xi = np1(nodes_xi, xi)
    np_eta = np1(nodes_eta, eta)
    np_sigma = np1(nodes_sigma, sigma)
    ep_xi = ep1(nodes_xi, xi)
    ep_eta = ep1(nodes_eta, eta)
    ep_sigma = ep1(nodes_sigma, sigma)
    return np.kron(np_sigma, np.kron(np_eta, ep_xi)), \
           np.kron(np_sigma, np.kron(ep_eta, np_xi)), \
           np.kron(ep_sigma, np.kron(np_eta, np_xi)),


def face_polynomials(nodes_xi, nodes_eta, nodes_sigma, xi, eta, sigma):
    """Compute the face polynomials built at ``nodes_xi``, ``nodes_eta``
    and evaluated at grid (``xi``, ``eta``, ``sigma``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param nodes_sigma: The nodes on which the 1d mimetic polynomials
        are built along the third axis.
    :param xi: Grid (``xi``, ``eta``, ``sigma``) is the grid to evaluate
        the polynomials.
    :param eta: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :param sigma: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :return: A tuple of three outputs:

        #. (numpy.array) The face polynomials `lee` evaluated at grid
            (``xi``, ``eta``, ``sigma``).
        #. (numpy.array) The face polynomials `ele` evaluated at grid
            (``xi``, ``eta``, ``sigma``).
        #. (numpy.array) The face polynomials `eel` evaluated at grid
            (``xi``, ``eta``, ``sigma``).

    :example:

    >>> nodes_xi, _ = lobatto(4)
    >>> nodes_eta, _ = lobatto(3)
    >>> nodes_sigma, _ = lobatto(2)
    >>> xi = np.linspace(-1, 1, 5)
    >>> eta = np.linspace(-1, 1, 6)
    >>> sigma = np.linspace(-1, 1, 7)
    >>> fps = face_polynomials(nodes_xi, nodes_eta, nodes_sigma, xi, eta, sigma)
    >>> print(fps[0].shape)
    (30, 210)
    >>> print(fps[1].shape)
    (32, 210)
    >>> print(fps[2].shape)
    (36, 210)
    >>> fps[0][0,:] # doctest: +ELLIPSIS
    array([ 4.50000000e+00, -5.27343750e-01,  0.00000000e+00, ...
    >>> fps[1][0,:] # doctest: +ELLIPSIS
    array([ 7.50000000e+00,  5.15625000e-01, -5.62500000e-01, ...
    >>> fps[2][0,:] # doctest: +ELLIPSIS
    array([ 1.50000000e+01,  1.03125000e+00, -1.12500000e+00, ...
    """
    np_xi = np1(nodes_xi, xi)
    np_eta = np1(nodes_eta, eta)
    np_sigma = np1(nodes_sigma, sigma)
    ep_xi = ep1(nodes_xi, xi)
    ep_eta = ep1(nodes_eta, eta)
    ep_sigma = ep1(nodes_sigma, sigma)
    return np.kron(ep_sigma, np.kron(ep_eta, np_xi)), \
           np.kron(ep_sigma, np.kron(np_eta, ep_xi)), \
           np.kron(np_sigma, np.kron(ep_eta, ep_xi)),


def volume_polynomials(nodes_xi, nodes_eta, nodes_sigma,
                       xi, eta, sigma):
    """Compute the volume polynomials built at ``nodes_xi``,
    ``nodes_eta`` and evaluated at grid (``xi``, ``eta``, ``sigma``).

    :param nodes_xi: The nodes on which the 1d mimetic polynomials are
        built along the first axis.
    :param nodes_eta: The nodes on which the 1d mimetic polynomials are
        built along the second axis.
    :param nodes_sigma: The nodes on which the 1d mimetic polynomials
        are built along the third axis.
    :param xi: Grid (``xi``, ``eta``, ``sigma``) is the grid to evaluate
        the polynomials.
    :param eta: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :param sigma: Grid (``xi``, ``eta``, ``sigma``) is the grid to
        evaluate the polynomials.
    :return: A 2d numpy.array whose 0-dimension refers to the number of
        node polynomials and 1-dimension refers to the values
        evaluated at grid (``xi``, ``eta``, ``sigma``).

    :example:

    >>> nodes_xi, _ = lobatto(2)
    >>> nodes_eta, _ = lobatto(3)
    >>> nodes_sigma, _ = lobatto(4)
    >>> xi = np.linspace(-1, 1, 5)
    >>> eta = np.linspace(-1, 1, 6)
    >>> sigma = np.linspace(-1, 1, 7)
    >>> vps = volume_polynomials(nodes_xi, nodes_eta, nodes_sigma, xi, eta, sigma)
    >>> print(vps.shape)
    (24, 210)
    >>> vps[0,:] # doctest: +ELLIPSIS
    array([ 2.25000000e+01,  1.50000000e+01,  7.50000000e+00,...
    """
    ep_xi = ep1(nodes_xi, xi)
    ep_eta = ep1(nodes_eta, eta)
    ep_sigma = ep1(nodes_sigma, sigma)
    return np.kron(ep_sigma, np.kron(ep_eta, ep_xi))

if __name__ == '__main__':
    import doctest
    doctest.testmod()