# -*- coding: utf-8 -*-
"""Numerical quadrature.

© mathischeap.com
"""

import numpy as np
from scipy.special import legendre

def _newton_method_(f, df_dx, x_0, n_max):
    """Newton method for root finding."""
    x = [x_0,]
    for i in range(n_max - 1):
        x.append(x[i] - f(x[i]) / df_dx(x[i]))
        if abs(x[i + 1] - x[i]) < np.finfo(float).eps * 10:
            break
    return x[-1]

def lobatto(p):
    """Compute the Lobatto quadrature nodes and weights using newton
    method. Credits belong to Lorenzo.

    :param p: The quadrature degree (positive integer).
    :type p: int
    :return: A tuple of two outputs:

        #. (numpy.ndarray) - The Lobatto quadrature nodes.
        #. (numpy.ndarray) - The Lobatto quadrature weights.

    :example:

    >>> n, w = lobatto(3)
    >>> n
    array([-1.       , -0.4472136,  0.4472136,  1.       ])
    >>> w
    array([0.16666667, 0.83333333, 0.83333333, 0.16666667])
    """
    x_0 = np.cos(np.arange(1, p) / p * np.pi)
    nodal_pts = np.zeros((p + 1))
    nodal_pts[0] = 1
    nodal_pts[-1] = -1

    legendre_prime = lambda x, n: (n * legendre(n - 1)(x) -
        n * x * legendre(n)(x)) / (1 - x ** 2)

    for i, ch_pt in enumerate(x_0):
        leg_p = lambda x: (1 - x**2)**2 * legendre_prime(x,p)
        leg_pp = lambda x: (2 * x * legendre_prime(x, p) -
            p * (p + 1) * legendre(p)(x)) * (1 - x ** 2)
        nodal_pts[i + 1] = _newton_method_(leg_p, leg_pp, ch_pt, 100)

    weights = 2 / (p * (p + 1) * (legendre(p)(nodal_pts))**2)
    return nodal_pts[::-1], weights

def gauss(p):
    """Compute the Gauss quadrature nodes and weights with fast and
    reliable numpy package.

    :param p: The quadrature degree (positive integer).
    :type p: int
    :return: A tuple of two outputs:

        #. (numpy.ndarray) - The Gauss quadrature nodes.
        #. (numpy.ndarray) - The Gauss quadrature weights.

    :Example:

    >>> n, w = gauss(3)
    >>> n
    array([-0.86113631, -0.33998104,  0.33998104,  0.86113631])
    >>> w
    array([0.34785485, 0.65214515, 0.65214515, 0.34785485])
    """
    return np.polynomial.legendre.leggauss(p+1)



if __name__ == "__main__":
    import doctest
    doctest.testmod()